"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const IORedis = require("ioredis");
class RedisDriver {
    constructor(serverUrl, options = {}) {
        this.serverUrl = serverUrl;
        this.options = options;
        const DEFAULT_OPTIONS = {
            retryStrategy(attempt) {
                // use exponential backoff
                // 50 (Min) => 100 => 200 => 400 => 800 => 1600 => 2000 (Max)
                return Math.min(50 * Math.pow(2, attempt), 2000);
            },
        };
        this.client = new IORedis(serverUrl, Object.assign(Object.assign({}, DEFAULT_OPTIONS), (options.ioredis || {})));
    }
    async touch(key, lifetime) {
        // @see https://redis.io/commands/expire#return-value
        // @type Integer reply
        //
        // 1 if the timeout was set.
        // 0 if key does not exist.
        const reply = await this.client.expire(key, lifetime);
        return reply === 1;
    }
    async get(key) {
        const response = await this.client.get(key);
        if (response === null) { // indicates given key does not exist
            return undefined;
        }
        try {
            return JSON.parse(response);
        }
        catch (e) {
            return response; // if failed to decode serialized value, return raw value instead.
        }
    }
    async getMulti(keys) {
        if (keys.length === 0) {
            return {};
        }
        const response = await this.client.mget(...keys);
        return keys.reduce((hash, key, index) => {
            const val = response[index];
            if (val === null) {
                // Indicate given key does not exist
                hash[key] = undefined;
            }
            else {
                try {
                    hash[key] = JSON.parse(val);
                }
                catch (e) {
                    hash[key] = val;
                }
            }
            return hash;
        }, {});
    }
    async set(key, value, lifetime) {
        const serialized = JSON.stringify(value);
        // @see https://redis.io/commands/setex#return-value
        // @type Simple string Reply
        let reply;
        if (!lifetime) {
            reply = await this.client.set(key, serialized);
        }
        else {
            reply = await this.client.setex(key, lifetime, serialized);
        }
        if (reply !== "OK") {
            throw new Error(`RedisDriver failed to set: '${key} - ${value}'`);
        }
    }
    async del(key) {
        // @see https://redis.io/commands/del#return-value
        // @type Integer reply: The number of keys that were removed.
        const reply = await this.client.del(key);
        return reply > 0;
    }
    async flush() {
        // @see https://redis.io/commands/flushdb
        // @type Simple string reply
        //
        // @note starting redis 4, redis support **ASYNC** flush mode.
        // since redis blocks everything during flush process,
        // flushing database which has huge number of keys can be cause redis server outage.
        // so it is great improvement, but currently AWS ElastiCache does not support redis 4.
        // @note this command does not flush entire redis database,
        // so this command **ONLY** flushes current active database
        const reply = await this.client.flushdb();
        if (reply !== "OK") {
            throw new Error(`RedisDriver failed to flush: '${reply}'`);
        }
    }
    async end() {
        await this.client.quit(); // quit method should disconnect connection cleanly
    }
}
exports.RedisDriver = RedisDriver;
//# sourceMappingURL=redis.js.map