"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto = require("crypto");
function isNotUndefined(x) {
    return x !== undefined;
}
class MemcachedFetcher {
    constructor(driver, options = {}) {
        this.driver = driver;
        const keyHashing = options.keyHashing !== undefined ? options.keyHashing : true;
        if (keyHashing) {
            this.keyHasher = (key) => crypto.createHash("md5")
                .update(key)
                .digest("hex");
        }
        else {
            this.keyHasher = (key) => key;
        }
    }
    async fetch(key, lifetime, fetcher) {
        const hash = this.keyHasher(key);
        let value = await this.driver.get(hash);
        if (!isNotUndefined(value)) {
            value = await fetcher();
            await this.driver.set(hash, value, lifetime);
        }
        return value;
    }
    async del(key) {
        const hash = this.keyHasher(key);
        return await this.driver.del(hash);
    }
    async multiFetch(args, key, lifetime, fetcher) {
        if (args.length === 0) {
            return [];
        }
        const { namespace, argToKey } = (() => {
            if (typeof key === "string") {
                return { namespace: key, argToKey: (arg) => String(arg) };
            }
            else {
                return { namespace: key[0], argToKey: key[1] };
            }
        })();
        const argsToKeyMap = new Map(args.map((arg) => {
            const hash = this.keyHasher(`${namespace}:${argToKey(arg).toString()}`);
            return [arg, hash];
        }));
        const cached = await this.driver.getMulti(Array.from(argsToKeyMap.values()));
        const missingArgs = args.filter((arg) => !isNotUndefined(cached[argsToKeyMap.get(arg)]));
        const fetchedArray = missingArgs.length > 0 ? await fetcher(missingArgs) : [];
        if (fetchedArray.length !== missingArgs.length) {
            throw new Error("Fetcher must return same length of result with Args.length");
        }
        const fetched = new Map(missingArgs.map((arg, index) => [arg, fetchedArray[index]]));
        await Promise.all(Array.from(fetched).map(([arg, result]) => {
            if (isNotUndefined(result)) {
                this.driver.set(argsToKeyMap.get(arg), result, lifetime);
            }
            else {
                // if fetcher returns undefined, that means user intentionally not want to cache this value
            }
        }));
        return args.map((arg) => {
            const hash = argsToKeyMap.get(arg);
            const value = cached[hash];
            if (isNotUndefined(value)) {
                return value;
            }
            else {
                return fetched.get(arg);
            }
        });
    }
    async multiFetchDelete(args, key) {
        const { namespace, argToKey } = (() => {
            if (typeof (key) === "string") {
                return { namespace: key, argToKey: (arg) => String(arg) };
            }
            else {
                return { namespace: key[0], argToKey: key[1] };
            }
        })();
        await Promise.all(args.map(async (arg) => {
            const hashedKey = this.keyHasher(`${namespace}:${argToKey(arg).toString()}`);
            await this.driver.del(hashedKey);
        }));
    }
}
exports.MemcachedFetcher = MemcachedFetcher;
//# sourceMappingURL=fetcher.js.map